<?php



/****************************************************************************************************************
 *	Here we hook the function to email group members to occur after any activity stream update for the group. 	*
 *	This simply does the notify stuff then returns the $params it was passed 								  	*
 ****************************************************************************************************************/
add_action( 'bp_activity_add' , 'ass_notify_group_members' , 50 );
// The actual email notification function
function ass_notify_group_members( $params ) {
	global $bp, $ass_activities;
		
	if ( $params['component'] == 'groups' && $params['type'] != 'created_group' && $params['type'] != 'deleted_wiki_comment' ) {
		$group_id = $params['item_id'];
		$activity_user_id = $bp->loggedin_user->id;
		// Check the age of the registration of the user vs the site setting for minimum registration time
		$ass_reg_age_setting = get_site_option( 'ass_activity_frequency_ass_registered_req' );
		if ( $ass_reg_age_setting != 'n/a' ) {
			$current_user_info = get_userdata( $activity_user_id );
			if ( strtotime(current_time("mysql", 0)) - strtotime($current_user_info->user_registered) < ( $ass_age_setting*24*60*60 ) ) {
				return $params;
			}
		}
		$activity_user_name = $bp->loggedin_user->fullname;
		$activity_link = $params['primary_link'];
		$activity_content = strip_tags(stripslashes($params['content']));
		$activity_type = $params['type'];		
		// Generate a nice description for the update based on the activity type.
		// First set a description based on the default
		$activity_nice_name = $ass_activities[0]['nice_name'];
		// Now change it if this activity type is defined
		foreach ( $ass_activities as $ass_activity ) {
			if ( $ass_activity['type'] == $activity_type ) {
				$activity_nice_name = $ass_activity['nice_name'];
			}
		}
		$group = new BP_Groups_Group( $group_id, false, true );
		$subject = '[' . get_blog_option( BP_ROOT_BLOG, 'blogname' ) . ' | ' . $group->name . '] ' . $activity_user_name . ' ' . $activity_nice_name . '.' ;
		
		$user_ids = BP_Groups_Member::get_group_member_ids( $group->id ); 
		foreach ( $user_ids as $user_id ) { 
			// Make sure the user has activity notification settings.  Set default if not.
			ass_user_settings( $user_id );
			// If update was done by the current user we don't bother to email them
			if ( $user_id == $activity_user_id ) continue;
			// Check to see if user has disabled updates for this activity type
			if ( 'no' == get_usermeta( $user_id, 'ass_new_' . $activity_type . '_' . $group_id ) ) continue;
			// Check to see if this update falls within the frequency limit set by user
			$ass_frequency = get_usermeta( $user_id, 'ass_new_' . $activity_type . '_' . $group_id );
			$ass_last_update = groups_get_groupmeta( $group_id, 'ass_new_' . $activity_type . '_' . $user_id  );
			if ( time() - $ass_last_update < $ass_frequency ) continue;
			
			// Set the group timestamp for this activity type and user id
			groups_update_groupmeta( $group_id , 'ass_new_' . $activity_type . '_' . $user_id , time() );
			
			// Get the details for the user
			$ud = bp_core_get_core_userdata( $user_id );

			// Set up and send the message
			$to = $ud->user_email;

			$group_link = $bp->root_domain . '/' . $bp->groups->slug . '/' . $group->slug;
			if ( $activity_link == '' ) {
				$activity_link = $group_link;
			}
			$settings_link = $group_link . '/notifications/';

			$message = sprintf( __(
'%s %s.

%s

You can view or respond to this activity by clicking on the link below:
%s

---------------------
', 'buddypress' ), $activity_user_name, $activity_nice_name, $activity_content, $activity_link );


			$message .= sprintf( __( 'To disable these notifications please log in and go to: %s', 'buddypress' ), $settings_link );

			// Send it
			wp_mail( $to, $subject, $message );

			unset( $message, $to );
		}
		
	}
	
	return $params;
}





























/****************************************************************************************************************
 *	Here we hook into the query_vars parsing so we can get our own form vars									*
 *																											  	*
 ****************************************************************************************************************/
function ass_form_vars($public_query_vars) {
	global $ass_activities;
		
	$public_query_vars[] = 'ass_form';
	$public_query_vars[] = 'ass_admin_notify';
	$public_query_vars[] = 'ass_group_id';
	$public_query_vars[] = 'ass_admin_notice';
	$public_query_vars[] = 'ass_admin_settings';
	$public_query_vars[] = 'ass_registered_req';
	
	foreach ( $ass_activities as $ass_activity ) {
		$public_query_vars[] = $ass_activity['type'];
	}

	return ($public_query_vars);
}
add_filter('query_vars', 'ass_form_vars');































/****************************************************************************************************************
 *	Call this at the start of our form processing function to get the variables for use in our script			*
 *																											  	*
 ****************************************************************************************************************/
function ass_get_form_vars(){  
    global $ass_form_vars, $ass_activities;  
	
    if(get_query_var('ass_form')) {  
        $ass_form_vars['ass_form'] = mysql_real_escape_string(get_query_var('ass_form'));  
    }
	
    if(get_query_var('ass_group_id')) {  
        $ass_form_vars['ass_group_id'] = mysql_real_escape_string(get_query_var('ass_group_id'));  
    }
		
    if(get_query_var('ass_admin_notify')) {  
        $ass_form_vars['ass_admin_notify'] = mysql_real_escape_string(get_query_var('ass_admin_notify'));  
	}
	
    if(get_query_var('ass_admin_notice')) {  
        $ass_form_vars['ass_admin_notice'] = mysql_real_escape_string(get_query_var('ass_admin_notice'));  
	}
	
    if(get_query_var('ass_admin_settings')) {  
        $ass_form_vars['ass_admin_settings'] = mysql_real_escape_string(get_query_var('ass_admin_settings'));  
	}
	
    if(get_query_var('ass_registered_req')) {  
        $ass_form_vars['ass_registered_req'] = mysql_real_escape_string(get_query_var('ass_registered_req'));  
	}
	
	foreach ( $ass_activities as $ass_activity ) {
		if(get_query_var($ass_activity['type'])) {  
			$ass_form_vars[$ass_activity['type']] = mysql_real_escape_string(get_query_var($ass_activity['type']));  
		}
	}
	
    return $ass_form_vars;  
} 
//-------------------------------------------------------------------------------------------















/****************************************************************************************************************
 *	This is called before a template_redirect so we can process our form and give the user feedback as needed	*
 *																											  	*
 ****************************************************************************************************************/
function ass_admin_notice() {
    global $bp, $wpdb, $ass_form_vars, $ass_activities;
	
	// Get the form vars
	ass_get_form_vars();  
	
	if ( $ass_form_vars['ass_admin_notice'] && $ass_form_vars['ass_admin_notify'] ) {
		// Check the nonce
		wp_verify_nonce('ass_admin_notice');
		
		// Make the group ID a little easier to read
		$group_id = $ass_form_vars['ass_group_id'];
		
		// Make sure the user is an admin or mod of this group
		if ( !groups_is_user_admin( $bp->loggedin_user->id , $group_id ) && !groups_is_user_mod( $bp->loggedin_user->id , $group_id ) ) {
			// If they're not, tell them so and then 'get the fudge out'
			bp_core_add_message( __( 'You are not allowed to do that.', 'buddypress' ), 'error' );
			bp_core_redirect( $bp->root_domain );
		}
		// Post an update to the group and force sending of an email to all group members
		$activity_user_id = $bp->loggedin_user->id;
		$activity_user_name = $bp->loggedin_user->fullname;
		$activity_link = '';
		$activity_content = strip_tags(stripslashes($ass_form_vars['ass_admin_notice']));
		$activity_type = 'admin_notice';		
		// Generate a nice description for the update based on the activity type.
		// First set a description based on the default
		$activity_nice_name = 'posted an important group update';
		
		$group = new BP_Groups_Group( $group_id, false, true );
		$subject = '[' . get_blog_option( BP_ROOT_BLOG, 'blogname' ) . ' | ' . $group->name . '] ' . $activity_user_name . ' ' . $activity_nice_name . '.' ;

		$user_ids = BP_Groups_Member::get_group_member_ids( $group->id ); 
		foreach ( $user_ids as $user_id ) { 

			// Get the details for the user
			$ud = bp_core_get_core_userdata( $user_id );

			// Set up and send the message
			$to = $ud->user_email;

			$group_link = $bp->root_domain . '/' . $bp->groups->slug . '/' . $group->slug;
			if ( $activity_link == '' ) {
				$activity_link = $group_link;
			}
			$settings_link = $group_link . '/notifications/';

			$message = sprintf( __(
'%s %s.

%s

Please respond to this notice by visiting your group homepage:
%s

---------------------
', 'buddypress' ), $activity_user_name, $activity_nice_name, $activity_content, $activity_link );

			// Send it
			wp_mail( $to, $subject, $message );

			unset( $message, $to );
		}
		bp_core_add_message( __( 'Message sent successfully', 'buddypress' ) );
		bp_core_redirect( wp_get_referer() );
	} 
	// If we get to this point the page request isn't our submitted form.  The rest of WP will load normally now
}
add_action('template_redirect', 'ass_admin_notice');  
//-------------------------------------------------------------------------------------------

























/****************************************************************************************************************
 *	This is called before a template_redirect so we can process our form and give the user feedback as needed	*
 *																											  	*
 ****************************************************************************************************************/
function ass_update_settings() {
    global $bp, $wpdb, $ass_form_vars, $ass_activities;
	
	// Get the form vars
	ass_get_form_vars();  
	
	if ( $ass_form_vars['ass_group_id'] && $ass_form_vars['ass_form'] ) {
		// Check the nonce
		wp_verify_nonce('ass_settings');
		
		// Make the group ID a little easier to read
		$group_id = $ass_form_vars['ass_group_id'];
		
		// Make sure the user is a member of this group
		if ( !groups_is_user_member( $bp->loggedin_user->id , $group_id ) ) {
			// If they're not, tell them so and then 'get the fudge out'
			bp_core_add_message( __( 'You are not allowed to do that.', 'buddypress' ), 'error' );
			bp_core_redirect( $bp->root_domain );
		}
		// Process each of the user setting changes
		foreach ( $ass_form_vars as $key=>$ass_form_var ) {
			// Check to get rid of the group_id and form submit check vars.  Oops.
			if ( $key == 'ass_group_id' || $key == 'ass_form' ) continue;
			
			if ( $ass_form_var > 0 && $ass_form_var < 24*60*60+1 ) {
				update_usermeta( $bp->loggedin_user->id, 'ass_new_' . $key . '_' . $group_id, $ass_form_var );  
			} else {
				update_usermeta( $bp->loggedin_user->id, 'ass_new_' . $key . '_' . $group_id, 'no' );  
			}
		}
		// End of user settings change process
		
		bp_core_add_message( __( 'Settings saved successfully', 'buddypress' ) );
		bp_core_redirect( wp_get_referer() );
	} 
	// If we get to this point the page request isn't our submitted form.  The rest of WP will load normally now
}
add_action('template_redirect', 'ass_update_settings');  
//-------------------------------------------------------------------------------------------
























/****************************************************************************************************************
 *	This one gets the settings for all the current possible activity types for the user_id user					*
 *	If there is nothing, the default is set																	  	*
 ****************************************************************************************************************/
function ass_user_settings( $user_id ) {
	global $bp, $ass_activities;
	
	foreach ( $ass_activities as $ass_activity ) {
		// Get the user meta for this activity type
		$user_setting = get_usermeta( $user_id , 'ass_new_' . $ass_activity['type'] . '_' . $bp->groups->current_group->id );
		// If the user doesn't have a setting, set default
		if ( !$user_setting ) {
			update_usermeta( $user_id , 'ass_new_' . $ass_activity['type'] . '_' . $bp->groups->current_group->id , $ass_activity['default_frequency'] );  
		}
		
	}
	
}



















/****************************************************************************************************************
 *	This returns a table of settings fields for all of the activity types available								*
 *	If there is nothing set for the current user and activity type, applies default 							*
 ****************************************************************************************************************/
function ass_group_notification_settings_fields( ) {
	global $bp, $ass_activities;
	
	ass_user_settings( $bp->loggedin_user->id );

	$output = '<table>';
	$output .= '<th>Activity Type</th><th>Maximum Frequency of Emails</th>';
	
	foreach ( $ass_activities as $ass_activity ) {
		// Get the user meta for this activity type
		$user_setting = get_usermeta( $bp->loggedin_user->id , 'ass_new_' . $ass_activity['type'] . '_' . $bp->groups->current_group->id );
		
		$output .= '<tr>';
		$output .= '<td><div class="activity-subscription-settings-field">';
		$output .= ucfirst($ass_activity['nice_name']); 
		$output .= '</div></td>';
		$output .= '<td class="activity-subscription-radio-cell">';
		$output .= '<input type="radio" name="' . $ass_activity['type'] . '" value="' . 1*60 . '"';
		if ($user_setting == 1*60) $output .= 'checked="checked"';
		$output .= '/>1 min';
		$output .= '<input type="radio" name="' . $ass_activity['type'] . '" value="' . 15*60 . '"';
		if ($user_setting == 15*60) $output .= 'checked="checked"';
		$output .= '/>15 mins';
		$output .= '<input type="radio" name="' . $ass_activity['type'] . '" value="' . 60*60 . '"';
		if ($user_setting == 60*60) $output .= 'checked="checked"';
		$output .= '/>60 mins';
		$output .= '<input type="radio" name="' . $ass_activity['type'] . '" value="' . 12*60*60 . '"'; 
		if ($user_setting == 12*60*60) $output .= 'checked="checked"';
		$output .= '/>12 hours';
		$output .= '<input type="radio" name="' . $ass_activity['type'] . '" value="' . 24*60*60 . '"'; 
		if ($user_setting == 24*60*60) $output .= 'checked="checked"';
		$output .= '/>24 hours';
		$output .= '<input type="radio" name="' . $ass_activity['type'] . '" value="no"';
		if ($user_setting == 'no') $output .= 'checked="checked"';
		$output .= '/>Never';
		$output .= '</td>';
	}	
	
	$output .= '</table>';
	
	return $output;
}


















/****************************************************************************************************************
 *	This function sets up the detault activity types.  This info is used to build meta name, nice name, etc		*
 *	Once things are setup, or if they already are, it sets the plugin constant								  	*
 ****************************************************************************************************************/
function ass_init() {
	global $ass_activities;
	
	if ( !get_site_option( 'ass_activity_frequency_ass_registered_req' ) ) update_site_option( 'ass_activity_frequency_ass_registered_req' , 3 );
	
	$ass_activities = array(	array("type"=>"default", "nice_name"=>"added something new to the group", "default_frequency"=>3600),
								array("type"=>"joined_group", "nice_name"=>"joined the group", "default_frequency"=>3600),
								array("type"=>"new_forum_topic", "nice_name"=>"created a new forum topic", "default_frequency"=>3600),
								array("type"=>"new_forum_post", "nice_name"=>"replied to a forum topic", "default_frequency"=>3600),
								array("type"=>"activity_update", "nice_name"=>"posted an update to the group", "default_frequency"=>3600)
							); 
	// action hook for people to add additional activity types
	do_action('ass_extra_activities');
	// update the default frequency in the array if the default frequency has been changed by site admin
	ass_update_activities();
}


















// Function to allow other plugins to add extra activity types.  Plugins should hook in with a higher priority than ass_update_activities
// ...as this will ensure that the default frequency settings are updated by the main functions
// Also updates the $ass_activities global with any new default frequency settings
function ass_update_activities() {
	global $ass_activities;
	// Update the default frequency settings for each activity
	foreach ( $ass_activities as &$ass_activity ) {
		$ass_activity['default_frequency'] = get_site_option( 'ass_activity_frequency_' . $ass_activity['type'] );
	}
}





// Functions to add the backend admin menu to control changing default settings
add_action('admin_menu', 'ass_admin_menu');
function ass_admin_menu() {
	add_submenu_page( 'bp-general-settings', "Group Notifications", "Group Notifications", 'manage_options', 'ass_admin_options', "ass_admin_options" );
}

function ass_admin_options() {
	global $bp;
	?>
	<div class="wrap">
		<h2>Group Notification Settings</h2>

		<form id="ass-admin-settings-form" method="post" action="<?php echo $bp->root_domain; ?>/?ass_admin_settings=1">
		
			<?php wp_nonce_field( 'ass_admin_settings' ); ?>
				
			<table class="form-table">
			<?php echo ass_admin_group_notification_settings_fields(); ?>
			</table>
			<br/>
			<p>To help protect against spam, you may wish to require a user to have been a member of the site for a certain amount of days before any group updates are emailed to the other group members.  By default, this is set to 3 days.  </p>
			Member must be registered for<input type="text" size="1" name="ass_registered_req" value="<?php echo get_site_option( 'ass_activity_frequency_ass_registered_req' ); ?>" style="text-align:center"/>days
			
			<p class="submit">
				<input type="submit" value="Save Settings" id="bp-admin-ass-submit" name="bp-admin-ass-submit" class="button-primary">
			</p>

		</form>
		
	</div>
	<?php
}






function ass_admin_group_notification_settings_fields( ) {
	global $bp, $ass_activities;
	
	$output .= '<th><h3>Activity Type</h3></th><th><h3>Default Frequency of Email Notifications</h3></th>';
	
	foreach ( $ass_activities as $ass_activity ) {
		// Get the user meta for this activity type
		$site_setting = get_site_option( 'ass_activity_frequency_' . $ass_activity['type'] );
		if ( $site_setting == '' ) {
			$site_setting = $ass_activity['default_frequency'];
			update_site_option( 'ass_activity_frequency_' . $ass_activity['type'] , $site_setting );
		}
		$output .= '<tr>';
		$output .= '<td>';
		$output .= ucfirst($ass_activity['nice_name']); 
		$output .= '</td>';
		$output .= '<td>';
		$output .= '<input type="radio" name="' . $ass_activity['type'] . '" value="' . 1*60 . '"';
		if ($site_setting == 1*60) $output .= 'checked="checked"';
		$output .= '/>1 min';
		$output .= '<input type="radio" name="' . $ass_activity['type'] . '" value="' . 15*60 . '"';
		if ($site_setting == 15*60) $output .= 'checked="checked"';
		$output .= '/>15 mins';
		$output .= '<input type="radio" name="' . $ass_activity['type'] . '" value="' . 60*60 . '"';
		if ($site_setting == 60*60) $output .= 'checked="checked"';
		$output .= '/>60 mins';
		$output .= '<input type="radio" name="' . $ass_activity['type'] . '" value="' . 12*60*60 . '"'; 
		if ($site_setting == 12*60*60) $output .= 'checked="checked"';
		$output .= '/>12 hours';
		$output .= '<input type="radio" name="' . $ass_activity['type'] . '" value="' . 24*60*60 . '"'; 
		if ($site_setting == 24*60*60) $output .= 'checked="checked"';
		$output .= '/>24 hours';
		$output .= '<input type="radio" name="' . $ass_activity['type'] . '" value="no"';
		if ($site_setting == 'no') $output .= 'checked="checked"';
		$output .= '/>Never';
		$output .= '</td></tr>';
	}	
	
	return $output;
}












/****************************************************************************************************************
 *	This is called before a template_redirect so we can process our form and give the user feedback as needed	*
 *																											  	*
 ****************************************************************************************************************/
function ass_update_admin_settings() {
    global $bp, $wpdb, $ass_form_vars, $ass_activities;
	
	// Get the form vars
	ass_get_form_vars();  
	
	if ( $ass_form_vars['ass_admin_settings'] ) {
		// Check the nonce
		wp_verify_nonce('ass_admin_settings');
				
		// Make sure the user is site admin
		if ( !is_site_admin() ) {
			// If they're not, tell them so and then 'get the fudge out'
			bp_core_add_message( __( 'You are not allowed to do that.', 'buddypress' ), 'error' );
			bp_core_redirect( $bp->root_domain );
		}
		
		// Process each of the user setting changes
		foreach ( $ass_form_vars as $key=>$ass_form_var ) {
			// Check to get rid of the ass_admin_settings from fields.  Oops.
			if ( $key == 'ass_admin_settings' ) continue;
			
			// Input field for member age of registration is dealt with slightly differently
			if ( $key == 'ass_registered_req' ) {
				if ( $ass_form_var > 0 ) {
					update_site_option( 'ass_activity_frequency_' . $key , $ass_form_var );
				} else {
					update_site_option( 'ass_activity_frequency_' . $key , 'n/a' );
				}
				continue;
			}
			
			if ( $ass_form_var > 0 && $ass_form_var < 24*60*60+1 ) {
				update_site_option( 'ass_activity_frequency_' . $key , $ass_form_var );
			} else {
				update_site_option( 'ass_activity_frequency_' . $key , 'no' );
			}
		}
		// End of user settings change process
		
		bp_core_add_message( __( 'Settings saved successfully', 'buddypress' ) );
		bp_core_redirect( wp_get_referer() );
	} 
	// If we get to this point the page request isn't our submitted form.  The rest of WP will load normally now
}
add_action('template_redirect', 'ass_update_admin_settings');  
//-------------------------------------------------------------------------------------------




?>